YUI.add('webui', function(Y) {
  var json_error_msg = "The ticket system returned data that could not be understood. " +
                       "Please try refreshing, or contact support if this problem persists.";
  var refresh_msg = "Refreshing data...";
  Y.webui = {
    url_prefix: "/api/v1/",
    refresh_interval: (parseInt(Y.one('#refresh_rate')) || 60) * 1000,
    add_refresh_div: function(callback) {
      var refresh = Y.Node.create('<div id="refresh"></div>');
      refresh.setHTML('<small>Auto refresh <input type="checkbox" checked="true"' +
        'id="refresh_enabled"></input> every <input type="text"' + 
        'id="refresh_rate" value="60" maxlength="3" size="2"/> seconds.</small>'
      );
      
      Y.one('div#main-content').append(refresh);

      var refresh_rate = Y.one("#refresh_rate");
      Y.webui.refresh_timeout = null;
      var refresh_enabled = Y.one("#refresh_enabled");
      Y.webui.refresh_enabled = refresh_enabled.get('checked');

      var onChange = function(e) {
        var value = refresh_rate.get('value');
        if (Y.webui.refresh_timeout) {
          clearTimeout(Y.webui.refresh_timeout);
        }
        var val = parseInt(value);
        if (val !== 0) {
          Y.webui.refresh_interval = val * 1000;
          callback();
        }
      };

      refresh_rate.on('change', onChange);
      refresh_enabled.on('change', function(e) {
        if (Y.webui.refresh_timeout) {
          clearTimeout(Y.webui.refresh_timeout);
        }

        Y.webui.refresh_enabled = refresh_enabled.get('checked');

        if (Y.webui.refresh_enabled) {
          callback();
        }
      });
      onChange();
    },
    get_ticket: function(ticket_id, elem) {
      var table = null;
      if (Y.webui.main_table) {
        Y.webui.main_table.showMessage("loading data...");
      }
      data = Y.io(Y.webui.url_prefix + "fullticket/" + ticket_id + "/", {
        on: {
          success: function(tx, r) {
            console.log("success");
            var data;

            try {
              data = Y.JSON.parse(r.responseText);
            }
            catch (e) {
              Y.webui.main_table.showMessage(json_error_msg);
              return;
            }

            var new_div =  Y.webui.ticket_detail_element(data);
            elem.setHTML(new_div);

          },
          failure: function(tx, r, e) {
            console.log("failure: ", tx, r, e);
          }
        }
      });
    },
    get_tickets: function() {
      container = Y.one('#container');
      if (Y.webui.main_table) {
        Y.webui.main_table.showMessage("loading data...");
      }
      data = Y.io(Y.webui.url_prefix + "opentickets/", {
        on: {
          success: function(tx, r) {
            console.log("success");
            var data;

            try {
              data = Y.JSON.parse(r.responseText);
            }
            catch (e) {
              Y.webui.main_table.showMessage(json_error_msg);
              return;
            }

            if (data.objects && data.objects.length !== 0) {
              for (var i = 0; i < data.objects.length; i++) {
                if (data.objects[i]) {
                  Y.webui.add_item(data.objects[i]);
                }
              }
              Y.webui.check_statuses();
            } 
            if (Y.webui.main_table.data.size() === 0) {
              Y.webui.main_table.showMessage("No results found");
            }
          },
          failure: function(tx, r, e) {
            console.log("failure: ", tx, r, e);
          }
        }
      });
    },
    add_subticket: function(data) {
      var main_tr = Y.Node.create("<tr>");
      main_tr.appendChild(Y.Node.create('<th>subticket</th>'));
      for (var key in data) {
        var value = data[key];

        var tr = main_tr.appendChild(Y.Node.create('<tr>'));
        tr.addClass('subticket');
        tr.appendChild(Y.Node.create('<th>' + key + '</th>'));
        tr.appendChild(Y.Node.create('<td>' + value + '</td>'));
      }

      main_tr.addClass('subticket');
      return main_tr;
    },
    make_subticket: function(data) {
      var div = Y.Node.create("<div class='subticket'></div>");

      var table = div.appendChild(Y.Node.create("<table>"));
      var hidden_fields = Y.Array(['id', 'resource_uri', 'current_workflow_step', 'status', 'source_package_upload']);

      for (var key in data) {
        // Skip hidden fields
        if (Y.Array.indexOf(hidden_fields, key) != -1) {
          continue;
        }

        var value = data[key];
        if (key == 'artifact') {
          var span = Y.webui.add_artifacts(data['artifact']);
          value = span.getHTML();
        }

        if (key == 'source_package_upload') {
          var span = Y.webui.sp_upload(data['source_package_upload']);
          value = span.getHTML();
        }

        var tr = table.appendChild(Y.Node.create('<tr>'));
        tr.appendChild(Y.Node.create('<th>' + key + ': </th>'));
        tr.appendChild(Y.Node.create('<td>' + value + '</td>'));
      }

      return div;
    },
    basename: function(path) {
      return path.replace(/^.*\//, "");
    },
    add_artifacts: function(data) {
      var span = Y.Node.create('<span></span>');
      for (var i = 0; i < data.length; i++) {
        span.appendChild(Y.webui.artifact_link(data[i]));
        span.appendChild(Y.Node.create("<br/>"));
      }

      return span;
    },
    artifact_link: function(data) {
      var name = Y.webui.basename(data['name']);
      var link = Y.Node.create("<a/>");
      link.set('href', data['reference']);
      link.set('text', name);

      return link;
    },
    sp_upload: function(data) {
      var span = Y.Node.create("<span></span>");

      if (data.sourcepackage) {
        span.setHTML(data.sourcepackage.name + " " + data.version);
      }

      return span;
    },
    ticket_detail_element: function(data) {
      var hidden_fields = Y.Array(['id', 'title', 'subticket', 'resource_uri', 'current_workflow_step', 'status', 'owner', 'description']);

      div = Y.Node.create("<div id='table_wrapper'/>");
      ticket_div = div.appendChild(Y.Node.create("<div class='ticket'></div>"));
      table = ticket_div.appendChild(Y.Node.create("<table></table>"));

      if (data['title']) {
        div.prepend(Y.Node.create('<h2>#'  + data['id'] + ' - ' + data['title'] + '</h2><h4><strong>Workflow Step:</strong> ' + data['current_workflow_step'] + ' <strong>Status:</strong> ' + data['status'] + '</h4><strong>Owner:</strong> ' + data['owner'] + '<br /><strong>Description:</strong> ' + data['description']));
      }

      for (var key in data) {
        var value = data[key];

        if (Y.Array.indexOf(hidden_fields, key) != -1) {
          continue;
        }

        if (key == "title" || key == "subticket") {
          continue;
        }

        if (key == 'artifact') {
          var span = Y.webui.add_artifacts(data['artifact']);
          value = span.getHTML();
        }
        key = key.replace("_", " ");
        if (value == null) {
          value = ""
        }
        tr = Y.Node.create('<tr></tr>');
        table.appendChild(tr);
        tr.appendChild(Y.Node.create('<th>' + key.charAt(0).toUpperCase() + key.slice(1) + ': </th>'));
        tr.appendChild(Y.Node.create('<td>' + value + '</td>'));
      }

      if (data['subticket']) {
        div.appendChild("<h3>Uploads</h3>");
        var subticket_count = data['subticket'].length;
        if (subticket_count > 0) {
          for (var i = 0; i < subticket_count; i++) {
            div.appendChild("<h4>" + data['subticket'][i]['source_package_upload']['sourcepackage']['name'] + " " + data['subticket'][i]['source_package_upload']['version'] + "</h4><div id='status'><strong>Workflow step:</strong> " + data['subticket'][i]['current_workflow_step'] + " <strong>Status:</strong> " + data['subticket'][i]['status'] + "</div>");
            var subticket = data['subticket'][i];

            var subt = Y.webui.make_subticket(subticket);

            div.appendChild(subt);
          }
        }
      }
      return div;
    },
    get_ticket_id: function() {
      var query = window.location.search.split('?')[1];

      var ticket_id = null;
      if (query) {
        var ob = Y.QueryString.parse(query);
        var ticket_id = ob.ticket_id;
      }

      return ticket_id;
    },
    update_ticket: function() {
      var ticket_id = Y.webui.get_ticket_id();

      if (ticket_id) {
        Y.webui.get_ticket(ticket_id, Y.one('#container'));
      }

      // we're done so allow more refreshes if needed.
      if (Y.webui.refresh_timeout) {
        clearTimeout(Y.webui.refresh_timeout);
        Y.webui.refresh_timeout = null;
      }

      if (Y.webui.refresh_enabled && !Y.webui.refresh_timeout && Y.webui.refresh_interval > 0) {
        Y.webui.refresh_timeout = setTimeout(Y.webui.update_ticket, Y.webui.refresh_interval);
      }
    },
    has_ticket: function(ticket_ids, id) {
      return ticket_ids[id] === true;
    },
    check_statuses: function() {
      var table = Y.one('#table');
      var rows = table.all('tr');
      var ticket_id_map = {};

      var table_data = Y.webui.main_table.get('data').toJSON();

      for (var i = 0; i < table_data.length; i++) {
        var record = table_data[i];
        var ticket_id = record['ticket_id'];

        if (record['status'] !== 'Completed' && record['current_workflow_step'] !== 'Completed') {
          if (ticket_id) {
            ticket_id_map[ticket_id] = true;
          }
        }
      }

      // Check for new tickets and add them to the table.
      if (Y.webui.main_table) {
        data = Y.io(Y.webui.url_prefix + "opentickets/", {
          on: {
            success: function(tx, r) {
              var data;

              try {
                data = Y.JSON.parse(r.responseText)['objects'];
              }
              catch (e) {
                Y.webui.main_table.showMessage(json_error_msg);
                return;
              }

              if (data && data.length !== 0) {
                for (var i = 0; i < data.length; i++) {
                  id = data[i]['id'];
                  if (!Y.webui.has_ticket(ticket_id_map, id)) {
                    Y.webui.add_item(data[i]);
                    ticket_id_map[id] = true;
                  }
                }
              }
            }
          }
        });
      }

      ticket_ids = Object.keys(ticket_id_map);
      if (ticket_ids.length > 0) {
        ids = ticket_ids.join(';');
        Y.webui.main_table.showMessage(refresh_msg);
        data = Y.io(Y.webui.url_prefix +  "ticketstatus/set/" + ids + "/", {
          on: {
            success: function(tx, r) {
              var data;

              try {
                data = Y.JSON.parse(r.responseText)['objects'];
              }
              catch (e) {
                Y.webui.main_table.showMessage(json_error_msg);
                return;
              }

              var data_map = {};
              for (var i = 0; i < data.length; i++) {
                data_map[data[i]['id']] = data[i];
              }

              var recordset = Y.webui.main_table.data.toJSON();

              for (var i = 0; i < recordset.length; i++) {

                var id = recordset[i]['ticket_id'];
                if (data_map[id]) {
                  recordset[i] = data_map[id];
                  recordset[i]['ticket_id'] = recordset[i]['id'];
                }
              }

              Y.webui.main_table.set('data', recordset);
              Y.webui.main_table.render();

              // we're done so allow more refreshes if needed.
              if (Y.webui.refresh_timeout) {
                clearTimeout(Y.webui.refresh_timeout);
                Y.webui.refresh_timeout = null;
              }
              if (Y.webui.refresh_enabled && !Y.webui.refresh_timeout && Y.webui.refresh_interval > 0) {
                Y.webui.refresh_timeout = setTimeout(Y.webui.check_statuses, Y.webui.refresh_interval);
              }
            },
            complete: function(tx) {
              if (Y.webui.refresh_enabled && !Y.webui.refresh_timeout && Y.webui.refresh_interval > 0) {
                Y.webui.refresh_timeout = setTimeout(Y.webui.check_statuses, Y.webui.refresh_interval);
              }
            }
          }
        });
      } else {
        if (Y.webui.refresh_timeout) {
          clearTimeout(Y.webui.refresh_timeout);
          Y.webui.refresh_timeout = null;
        }
        if (Y.webui.refresh_enabled && !Y.webui.refresh_timeout && Y.webui.refresh_interval > 0) {
          Y.webui.refresh_timeout = setTimeout(Y.webui.check_statuses, Y.webui.refresh_interval);
        }
      }
    },
    add_item: function(item) {
      var entry = item;
      var id = entry['id'];
      var table = Y.webui.main_table;

      if (table) {
        var tr = table.addRow({
          title: entry['title'],
          current_workflow_step: entry['current_workflow_step'],
          status: entry['status'],
          ticket_id: id
        });
      }
    },
    make_main_table: function(div_id) {

      var format_id = function(o) {
        o.rowClass = "data-row-" + o.data.ticket_id;

        return o.value;
      };
      var columns = [
        {key: 'title', label: 'Title', formatter: format_id},
        {key: 'current_workflow_step', label: 'Workflow step'},
        {key: 'status', label: 'Status'}
      ]
      var data = [];

      var main_table = new Y.DataTable({
        columnset: columns,
        recordset: data
      });
      main_table.render(div_id);

      Y.webui.main_table = main_table;
    }
  };
}, '0.0.1', {requires: ['io', 'node', 'json-parse', 'datatable']});
